/**
 * Assignment #8: POSIX File System Functions
 * Sample Solution
 *
 * Instructor: Ron Mak
 * CMPE 142 Operating Systems
 * Computer Engineering Department
 * San Jose State University
 */
#ifndef PARENT_H_
#define PARENT_H_

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "globals.h"
#include "Watcher.h"

using namespace std;

class Parent
{
public:
    Parent(int file_ids[])
    {
        int read_fid = file_ids[READ_FID];
        close(file_ids[WRITE_FID]);

        // Give the child process a chance to start.
        sleep(1);

        Watcher watch(read_fid);

        watch("mkdir subdir1",
              [] () { mkdir("subdir1", S_IRWXU | S_IRWXG
                                     | S_IROTH | S_IXOTH); });

        watch("mkdir subdir2",
              [] () { mkdir("subdir2", S_IRWXU | S_IRWXG
                                     | S_IROTH | S_IXOTH); });

        watch("cd subdir1", [] () { chdir("subdir1"); });

        watch("open file1.txt, append, close",
              [] () {
                        ofstream file("file1.txt", ios::app);

                        for (int i = 0; i < 5; i++)
                        {
                            file << "Hello, world!" << endl;
                        }

                        file.close();
                    });

        watch("cp file1.txt file2.txt",
              [] () {
                        ifstream from("file1.txt", ios::in);
                        ofstream   to("file2.txt", ios::out);

                        // Copy the file.
                        to << from.rdbuf();

                        from.close();
                        to.close();
                    });

        watch("mv file2.txt ../subdir2/file2.txt",
              [] () { rename("file2.txt", "../subdir2/file2.txt"); });

        watch("cd ../subdir2", [] () { chdir("../subdir2"); });
        watch("ln file2.txt file3.txt",
              [] () { link("file2.txt", "file3.txt"); });
        watch("ln -s file2.txt file2s.txt",
              [] () { symlink("file2.txt", "file2s.txt"); });

        watch("cd ../subdir1", [] () { chdir("../subdir1"); });
        watch("rm file1.txt",  [] () { unlink("file1.txt"); });

        watch("cd ..", [] () { chdir(".."); });
        watch("rmdir subdir1",  [] () { rmdir("subdir1"); });

        watch("cd subdir2", [] () { chdir("subdir2"); });
        watch("chmod a+rw file2.txt",
              [] () { chmod("file2.txt", S_IRUSR | S_IWUSR
                                       | S_IRGRP | S_IWGRP
                                       | S_IROTH | S_IWOTH); });

        watch("cat file2s.txt",
              [] () {
                        ifstream from("file2s.txt", ios::in);
                        cout << from.rdbuf();

                        from.close();
                    });
    }
};

#endif /* PARENT_H_ */
